@@ -33,21 +33,34 @@
 
 #include "gdm-common.h"
 
-void
-gdm_set_fatal_warnings_if_unstable (void)
+gboolean
+gdm_is_version_unstable (void)
 {
-        char **versions;
+        char   **versions;
+        gboolean unstable;
+
+        unstable = FALSE;
 
         versions = g_strsplit (VERSION, ".", 3);
         if (versions && versions [0] && versions [1]) {
                 int major;
                 major = atoi (versions [1]);
                 if ((major % 2) != 0) {
-                        g_setenv ("G_DEBUG", "fatal_criticals", FALSE);
-                        g_log_set_always_fatal (G_LOG_LEVEL_CRITICAL);
+                        unstable = TRUE;
                 }
         }
         g_strfreev (versions);
+
+        return unstable;
+}
+
+void
+gdm_set_fatal_warnings_if_unstable (void)
+{
+        if (gdm_is_version_unstable ()) {
+                g_setenv ("G_DEBUG", "fatal_criticals", FALSE);
+                g_log_set_always_fatal (G_LOG_LEVEL_CRITICAL);
+        }
 }
 
 int
@@ -119,6 +132,11 @@ gdm_string_hex_encode (const GString *source,
         const unsigned char *end;
         gboolean             retval;
 
+        g_return_val_if_fail (source != NULL, FALSE);
+        g_return_val_if_fail (dest != NULL, FALSE);
+        g_return_val_if_fail (source != dest, FALSE);
+        g_return_val_if_fail (start >= 0, FALSE);
+        g_return_val_if_fail (dest >= 0, FALSE);
         g_assert (start <= source->len);
 
         result = g_string_new (NULL);
@@ -166,6 +184,12 @@ gdm_string_hex_decode (const GString *source,
         gboolean             retval;
         gboolean             high_bits;
 
+        g_return_val_if_fail (source != NULL, FALSE);
+        g_return_val_if_fail (dest != NULL, FALSE);
+        g_return_val_if_fail (source != dest, FALSE);
+        g_return_val_if_fail (start >= 0, FALSE);
+        g_return_val_if_fail (dest >= 0, FALSE);
+
         g_assert (start <= source->len);
 
         result = g_string_new (NULL);
@@ -272,3 +296,116 @@ gdm_string_hex_decode (const GString *source,
 
         return retval;
 }
+
+static gboolean
+_fd_is_character_device (int fd)
+{
+        struct stat file_info;
+
+        if (fstat (fd, &file_info) < 0) {
+                return FALSE;
+        }
+
+        return S_ISCHR (file_info.st_mode);
+}
+
+static gboolean
+_read_bytes (int      fd,
+             char    *bytes,
+             gsize    number_of_bytes,
+             GError **error)
+{
+        size_t bytes_left_to_read;
+        size_t total_bytes_read = 0;
+        gboolean premature_eof;
+
+        bytes_left_to_read = number_of_bytes;
+        premature_eof = FALSE;
+        do {
+                size_t bytes_read = 0;
+
+                bytes_read = read (fd, ((guchar *) bytes) + total_bytes_read,
+                                   bytes_left_to_read);
+
+                if (bytes_read > 0) {
+                        total_bytes_read += bytes_read;
+                        bytes_left_to_read -= bytes_read;
+                } else if (bytes_read == 0) {
+                        premature_eof = TRUE;
+                        break;
+                } else if ((errno != EINTR)) {
+                        break;
+                }
+        } while (bytes_left_to_read > 0);
+
+        if (premature_eof) {
+                g_set_error (error,
+                             G_FILE_ERROR,
+                             g_file_error_from_errno (ENODATA),
+                             g_strerror (ENODATA));
+
+                return FALSE;
+        } else if (bytes_left_to_read > 0) {
+                g_set_error (error,
+                             G_FILE_ERROR,
+                             g_file_error_from_errno (errno),
+                             g_strerror (errno));
+                return FALSE;
+        }
+
+        return TRUE;
+}
+
+/**
+ * Pulls a requested number of bytes from /dev/urandom
+ *
+ * @param size number of bytes to pull
+ * @param error error if read fails
+ * @returns The requested number of random bytes or #NULL if fail
+ */
+
+char *
+gdm_generate_random_bytes (gsize    size,
+                           GError **error)
+{
+        int fd;
+        char *bytes;
+        GError *read_error;
+
+        /* We don't use the g_rand_* glib apis because they don't document
+         * how much entropy they are seeded with, and it might be less
+         * than the passed in size.
+         */
+
+        fd = open ("/dev/urandom", O_RDONLY);
+
+        if (fd < 0) {
+                g_set_error (error,
+                             G_FILE_ERROR,
+                             g_file_error_from_errno (errno),
+                             "%s", g_strerror (errno));
+                close (fd);
+                return NULL;
+        }
+
+        if (!_fd_is_character_device (fd)) {
+                g_set_error (error,
+                             G_FILE_ERROR,
+                             g_file_error_from_errno (ENODEV),
+                             _("/dev/urandom is not a character device"));
+                close (fd);
+                return NULL;
+        }
+
+        bytes = g_malloc (size);
+        read_error = NULL;
+        if (!_read_bytes (fd, bytes, size, &read_error)) {
+                g_propagate_error (error, read_error);
+                g_free (bytes);
+                close (fd);
+                return NULL;
+        }
+
+        close (fd);
+        return bytes;
+}
