@@ -769,6 +769,13 @@ _gdm_server_set_user_name (GdmServer  *server,
         server->priv->user_name = g_strdup (name);
 }
 
+static void
+_gdm_server_set_disable_tcp (GdmServer  *server,
+                             gboolean    disabled)
+{
+        server->priv->disable_tcp = disabled;
+}
+
 static void
 gdm_server_set_property (GObject      *object,
                          guint         prop_id,
@@ -789,6 +796,9 @@ gdm_server_set_property (GObject      *object,
         case PROP_USER_NAME:
                 _gdm_server_set_user_name (self, g_value_get_string (value));
                 break;
+        case PROP_DISABLE_TCP:
+                _gdm_server_set_disable_tcp (self, g_value_get_boolean (value));
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -819,6 +829,9 @@ gdm_server_get_property (GObject    *object,
         case PROP_USER_NAME:
                 g_value_set_string (value, self->priv->user_name);
                 break;
+        case PROP_DISABLE_TCP:
+                g_value_set_boolean (value, self->priv->disable_tcp);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -831,9 +844,6 @@ gdm_server_constructor (GType                  type,
                         GObjectConstructParam *construct_properties)
 {
         GdmServer      *server;
-        GdmServerClass *klass;
-
-        klass = GDM_SERVER_CLASS (g_type_class_peek (GDM_TYPE_SERVER));
 
         server = GDM_SERVER (G_OBJECT_CLASS (gdm_server_parent_class)->constructor (type,
                                                                                     n_construct_properties,
@@ -915,6 +925,13 @@ gdm_server_class_init (GdmServerClass *klass)
                                                               "user name",
                                                               NULL,
                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+        g_object_class_install_property (object_class,
+                                         PROP_DISABLE_TCP,
+                                         g_param_spec_boolean ("disable-tcp",
+                                                               NULL,
+                                                               NULL,
+                                                               TRUE,
+                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
 }
 
