@@ -131,9 +131,37 @@ send_dbus_string_signal (GdmSessionRelay *session_relay,
         dbus_message_unref (message);
 }
 
+static void
+send_dbus_string_string_signal (GdmSessionRelay *session_relay,
+                                const char      *name,
+                                const char      *text1,
+                                const char      *text2)
+{
+        DBusMessage    *message;
+        DBusMessageIter iter;
+
+        g_return_if_fail (session_relay != NULL);
+
+        g_debug ("GdmSessionRelay: sending signal %s", name);
+        message = dbus_message_new_signal (GDM_SESSION_RELAY_DBUS_PATH,
+                                           GDM_SESSION_RELAY_DBUS_INTERFACE,
+                                           name);
+
+        dbus_message_iter_init_append (message, &iter);
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &text1);
+        dbus_message_iter_init_append (message, &iter);
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &text2);
+
+        if (! send_dbus_message (session_relay->priv->session_connection, message)) {
+                g_debug ("GdmSessionRelay: Could not send %s signal", name);
+        }
+
+        dbus_message_unref (message);
+}
+
 static void
 send_dbus_void_signal (GdmSessionRelay *session_relay,
-                       const char        *name)
+                       const char      *name)
 {
         DBusMessage    *message;
 
@@ -166,18 +194,20 @@ gdm_session_relay_close (GdmSession *session)
 }
 
 static void
-gdm_session_relay_setup (GdmSession *session)
+gdm_session_relay_setup (GdmSession *session,
+                         const char *service_name)
 {
         GdmSessionRelay *impl = GDM_SESSION_RELAY (session);
-        send_dbus_void_signal (impl, "Setup");
+        send_dbus_string_signal (impl, "Setup", service_name);
 }
 
 static void
 gdm_session_relay_setup_for_user (GdmSession *session,
+                                  const char *service_name,
                                   const char *username)
 {
         GdmSessionRelay *impl = GDM_SESSION_RELAY (session);
-        send_dbus_string_signal (impl, "SetupForUser", username);
+        send_dbus_string_string_signal (impl, "SetupForUser", service_name, username);
 }
 
 static void
@@ -742,15 +772,9 @@ do_introspect (DBusConnection *connection,
                                "    </signal>\n"
                                "    <signal name=\"Setup\">\n"
                                "      <arg name=\"service_name\" type=\"s\"/>\n"
-                               "      <arg name=\"x11_display_name\" type=\"s\"/>\n"
-                               "      <arg name=\"display_device\" type=\"s\"/>\n"
-                               "      <arg name=\"hostname\" type=\"s\"/>\n"
                                "    </signal>\n"
                                "    <signal name=\"SetupForUser\">\n"
                                "      <arg name=\"service_name\" type=\"s\"/>\n"
-                               "      <arg name=\"x11_display_name\" type=\"s\"/>\n"
-                               "      <arg name=\"display_device\" type=\"s\"/>\n"
-                               "      <arg name=\"hostname\" type=\"s\"/>\n"
                                "      <arg name=\"username\" type=\"s\"/>\n"
                                "    </signal>\n"
                                "    <signal name=\"Authenticate\">\n"
@@ -1017,10 +1041,6 @@ gdm_session_relay_set_property (GObject      *object,
                                 const GValue *value,
                                 GParamSpec   *pspec)
 {
-        GdmSessionRelay *self;
-
-        self = GDM_SESSION_RELAY (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1034,10 +1054,6 @@ gdm_session_relay_get_property (GObject    *object,
                                 GValue     *value,
                                 GParamSpec *pspec)
 {
-        GdmSessionRelay *self;
-
-        self = GDM_SESSION_RELAY (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1051,9 +1067,6 @@ gdm_session_relay_constructor (GType                  type,
                                GObjectConstructParam *construct_properties)
 {
         GdmSessionRelay      *session_relay;
-        GdmSessionRelayClass *klass;
-
-        klass = GDM_SESSION_RELAY_CLASS (g_type_class_peek (GDM_TYPE_SESSION_RELAY));
 
         session_relay = GDM_SESSION_RELAY (G_OBJECT_CLASS (gdm_session_relay_parent_class)->constructor (type,
                                                                                                             n_construct_properties,
