@@ -46,6 +46,16 @@
 #include "gdm-session-worker.h"
 #include "gdm-marshal.h"
 
+#if defined (HAVE_ADT)
+#include "gdm-session-solaris-auditor.h"
+#elif defined (HAVE_LIBAUDIT)
+#include "gdm-session-linux-auditor.h"
+#else
+#include "gdm-session-auditor.h"
+#endif
+
+#include "gdm-session-settings.h"
+
 #define GDM_SESSION_WORKER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_SESSION_WORKER, GdmSessionWorkerPrivate))
 
 #define GDM_SESSION_DBUS_PATH         "/org/gnome/DisplayManager/Session"
@@ -94,6 +104,7 @@ struct GdmSessionWorkerPrivate
         /* from Setup */
         char             *service;
         char             *x11_display_name;
+        char             *x11_authority_file;
         char             *display_device;
         char             *hostname;
         char             *username;
@@ -104,10 +115,14 @@ struct GdmSessionWorkerPrivate
         char            **arguments;
         GHashTable       *environment;
         guint32           cancelled : 1;
+        guint32           timed_out : 1;
         guint             state_change_idle_id;
 
         char             *server_address;
         DBusConnection   *connection;
+
+        GdmSessionAuditor  *auditor;
+        GdmSessionSettings *user_settings;
 };
 
 enum {
@@ -149,8 +164,9 @@ script_execute (const gchar *file,
         /* Count the arguments.  */
         int argc = 0;
 
-        while (argv[argc])
+        while (argv[argc]) {
                 ++argc;
+        }
 
         /* Construct an argument list for the shell.  */
         {
@@ -166,10 +182,11 @@ script_execute (const gchar *file,
                 }
 
                 /* Execute the shell. */
-                if (envp)
+                if (envp) {
                         execve (new_argv[0], new_argv, envp);
-                else
+                } else {
                         execv (new_argv[0], new_argv);
+                }
 
                 g_free (new_argv);
         }
@@ -179,8 +196,9 @@ static char *
 my_strchrnul (const char *str, char c)
 {
         char *p = (char*) str;
-        while (*p && (*p != c))
+        while (*p && (*p != c)) {
                 ++p;
+        }
 
         return p;
 }
@@ -200,13 +218,15 @@ gdm_session_execute (const char *file,
 
         if (!search_path || strchr (file, '/') != NULL) {
                 /* Don't search when it contains a slash. */
-                if (envp)
+                if (envp) {
                         execve (file, argv, envp);
-                else
+                } else {
                         execv (file, argv);
+                }
 
-                if (errno == ENOEXEC)
+                if (errno == ENOEXEC) {
                         script_execute (file, argv, envp, FALSE);
+                }
         } else {
                 gboolean got_eacces = 0;
                 const char *path, *p;
@@ -470,6 +490,49 @@ gdm_session_worker_get_username (GdmSessionWorker  *worker,
         return FALSE;
 }
 
+static void
+attempt_to_load_user_settings_as_root (GdmSessionWorker *worker,
+                                       const char       *username)
+{
+        struct passwd *passwd_entry;
+        uid_t          old_uid;
+        gid_t          old_gid;
+
+        old_uid = geteuid ();
+        old_gid = getegid ();
+
+        g_assert (old_uid == 0);
+        g_assert (old_gid == 0);
+
+        passwd_entry = getpwnam (username);
+
+        /* User input isn't a valid username
+         */
+        if (passwd_entry == NULL) {
+                return;
+        }
+
+        setegid (passwd_entry->pw_gid);
+        seteuid (passwd_entry->pw_uid);
+
+        /* FIXME: if this call fails, we should probably do some
+         * heuristics to try to figure out if .dmrc doesn't
+         * exist at all or just isn't readable yet.  If it doesn't
+         * exist at all (and won't exist by the end of the pam
+         * conversation) then we should send some sort of
+         * "NoSavedSettings" message to the slave so that it can
+         * tell the greeter to use specific default settings
+         * instead of the place holder "Last Session" "Last
+         * Language" items in the combo boxes in the panel
+         */
+        gdm_session_settings_load (worker->priv->user_settings,
+                                   passwd_entry->pw_dir,
+                                   NULL);
+
+        seteuid (old_uid);
+        setegid (old_gid);
+}
+
 static void
 gdm_session_worker_update_username (GdmSessionWorker *worker)
 {
@@ -479,6 +542,22 @@ gdm_session_worker_update_username (GdmSessionWorker *worker)
         username = NULL;
         res = gdm_session_worker_get_username (worker, &username);
         if (res) {
+                g_debug ("GdmSessionWorker: old-username='%s' new-username='%s'",
+                         username != NULL ? username : "<unset>",
+                         worker->priv->username != NULL ? worker->priv->username : "<unset>");
+
+                gdm_session_auditor_set_username (worker->priv->auditor, worker->priv->username);
+
+                /* We have a new username to try. If we haven't been able to
+                 * read user settings up until now, then give it a go now
+                 * (see the comment in do_setup for rationale on why it's useful
+                 * to keep trying to read settings)
+                 */
+                if (username != NULL &&
+                    !gdm_session_settings_is_loaded (worker->priv->user_settings)) {
+                        attempt_to_load_user_settings_as_root (worker, username);
+                }
+
                 if ((worker->priv->username == username) ||
                     ((worker->priv->username != NULL) && (username != NULL) &&
                      (strcmp (worker->priv->username, username) == 0)))
@@ -537,8 +616,10 @@ send_question_method (GdmSessionWorker *worker,
         dbus_message_unref (message);
 
         if (dbus_error_is_set (&error)) {
-                if (strcmp (error.name, GDM_SESSION_DBUS_ERROR_CANCEL) == 0) {
+                if (dbus_error_has_name (&error, GDM_SESSION_DBUS_ERROR_CANCEL)) {
                         worker->priv->cancelled = TRUE;
+                } else if (dbus_error_has_name (&error, DBUS_ERROR_NO_REPLY)) {
+                        worker->priv->timed_out = TRUE;
                 }
                 g_debug ("%s %s raised: %s\n",
                          method,
@@ -645,6 +726,7 @@ gdm_session_worker_process_pam_message (GdmSessionWorker          *worker,
         utf8_msg = convert_to_utf8 (query->msg);
 
         worker->priv->cancelled = FALSE;
+        worker->priv->timed_out = FALSE;
 
         user_answer = NULL;
         res = FALSE;
@@ -662,10 +744,15 @@ gdm_session_worker_process_pam_message (GdmSessionWorker          *worker,
                 res = gdm_session_worker_report_problem (worker, utf8_msg);
                 break;
         default:
-                g_debug ("GdmSessionWorker: unknown query of type %u\n", query->msg_style);
+                g_assert_not_reached ();
                 break;
         }
 
+        if (worker->priv->timed_out) {
+                send_dbus_void_method (worker->priv->connection, "CancelPendingQuery");
+                worker->priv->timed_out = FALSE;
+        }
+
         if (user_answer != NULL) {
                 /* we strdup and g_free to make sure we return malloc'd
                  * instead of g_malloc'd memory
@@ -674,6 +761,7 @@ gdm_session_worker_process_pam_message (GdmSessionWorker          *worker,
                         *response_text = strdup (user_answer);
                 }
 
+                memset (user_answer, '\0', strlen (user_answer));
                 g_free (user_answer);
 
                 g_debug ("GdmSessionWorker: trying to get updated username");
@@ -725,6 +813,8 @@ gdm_session_worker_pam_new_messages_handler (int                        number_o
                                                                        messages[i],
                                                                        &response_text);
                 if (!got_response) {
+                        memset (response_text, '\0', strlen (response_text));
+                        g_free (response_text);
                         goto out;
                 }
 
@@ -758,6 +848,32 @@ gdm_session_worker_pam_new_messages_handler (int                        number_o
         return return_value;
 }
 
+static void
+gdm_session_worker_start_auditor (GdmSessionWorker *worker)
+{
+
+/* FIXME: it may make sense at some point to keep a list of
+ * auditors, instead of assuming they are mutually exclusive
+ */
+#if defined (HAVE_ADT)
+        worker->priv->auditor = gdm_session_solaris_auditor_new (worker->priv->hostname,
+                                                                 worker->priv->display_device);
+#elif defined (HAVE_LIBAUDIT)
+        worker->priv->auditor = gdm_session_linux_auditor_new (worker->priv->hostname,
+                                                               worker->priv->display_device);
+#else
+        worker->priv->auditor = gdm_session_auditor_new (worker->priv->hostname,
+                                                         worker->priv->display_device);
+#endif
+}
+
+static void
+gdm_session_worker_stop_auditor (GdmSessionWorker *worker)
+{
+        g_object_unref (worker->priv->auditor);
+        worker->priv->auditor = NULL;
+}
+
 static void
 gdm_session_worker_uninitialize_pam (GdmSessionWorker *worker,
                                      int               status)
@@ -773,11 +889,19 @@ gdm_session_worker_uninitialize_pam (GdmSessionWorker *worker,
 
         if (worker->priv->state >= GDM_SESSION_WORKER_STATE_SESSION_OPENED) {
                 pam_close_session (worker->priv->pam_handle, 0);
+                gdm_session_auditor_report_logout (worker->priv->auditor);
+        } else {
+                gdm_session_auditor_report_login_failure (worker->priv->auditor,
+                                                          status,
+                                                          pam_strerror (worker->priv->pam_handle, status));
         }
 
         pam_end (worker->priv->pam_handle, status);
         worker->priv->pam_handle = NULL;
 
+        gdm_session_worker_stop_auditor (worker);
+
+        g_debug ("GdmSessionWorker: state NONE");
         worker->priv->state = GDM_SESSION_WORKER_STATE_NONE;
 }
 
@@ -787,8 +911,9 @@ _get_tty_for_pam (const char *x11_display_name,
 {
 #ifdef __sun
         return g_strdup (display_device);
-#endif
+#else
         return g_strdup (x11_display_name);
+#endif
 }
 
 static gboolean
@@ -797,6 +922,7 @@ gdm_session_worker_initialize_pam (GdmSessionWorker *worker,
                                    const char       *username,
                                    const char       *hostname,
                                    const char       *x11_display_name,
+                                   const char       *x11_authority_file,
                                    const char       *display_device,
                                    GError          **error)
 {
@@ -811,13 +937,14 @@ gdm_session_worker_initialize_pam (GdmSessionWorker *worker,
         pam_conversation.conv = (GdmSessionWorkerPamNewMessagesFunc) gdm_session_worker_pam_new_messages_handler;
         pam_conversation.appdata_ptr = worker;
 
+        gdm_session_worker_start_auditor (worker);
         error_code = pam_start (service,
                                 username,
                                 &pam_conversation,
                                 &worker->priv->pam_handle);
 
         if (error_code != PAM_SUCCESS) {
-                g_debug ("GdmSessionWorker: could not initialize pam");
+                g_debug ("GdmSessionWorker: could not initialize PAM");
                 /* we don't use pam_strerror here because it requires a valid
                  * pam handle, and if pam_start fails pam_handle is undefined
                  */
@@ -828,7 +955,7 @@ gdm_session_worker_initialize_pam (GdmSessionWorker *worker,
                              error_code == PAM_ABORT? _("general failure") :
                              error_code == PAM_BUF_ERR? _("out of memory") :
                              error_code == PAM_SYSTEM_ERR? _("application programmer error") :
-                             _("unscoped error"));
+                             _("unknown error"));
 
                 goto out;
         }
@@ -875,7 +1002,7 @@ gdm_session_worker_initialize_pam (GdmSessionWorker *worker,
                 goto out;
         }
 
-        g_debug ("GdmSessionWorker: initialized");
+        g_debug ("GdmSessionWorker: state SETUP_COMPLETE");
         worker->priv->state = GDM_SESSION_WORKER_STATE_SETUP_COMPLETE;
 
  out:
@@ -916,6 +1043,7 @@ gdm_session_worker_authenticate_user (GdmSessionWorker *worker,
                 goto out;
         }
 
+        g_debug ("GdmSessionWorker: state AUTHENTICATED");
         worker->priv->state = GDM_SESSION_WORKER_STATE_AUTHENTICATED;
 
  out:
@@ -949,9 +1077,16 @@ gdm_session_worker_authorize_user (GdmSessionWorker *worker,
 
         /* it's possible that the user needs to change their password or pin code
          */
-        if (error_code == PAM_NEW_AUTHTOK_REQD)
+        if (error_code == PAM_NEW_AUTHTOK_REQD) {
                 error_code = pam_chauthtok (worker->priv->pam_handle, PAM_CHANGE_EXPIRED_AUTHTOK);
 
+                if (error_code != PAM_SUCCESS) {
+                        gdm_session_auditor_report_password_change_failure (worker->priv->auditor);
+                } else {
+                        gdm_session_auditor_report_password_changed (worker->priv->auditor);
+                }
+        }
+
         if (error_code != PAM_SUCCESS) {
                 g_debug ("GdmSessionWorker: user is not authorized to log in: %s",
                          pam_strerror (worker->priv->pam_handle, error_code));
@@ -962,6 +1097,7 @@ gdm_session_worker_authorize_user (GdmSessionWorker *worker,
                 goto out;
         }
 
+        g_debug ("GdmSessionWorker: state AUTHORIZED");
         worker->priv->state = GDM_SESSION_WORKER_STATE_AUTHORIZED;
 
  out:
@@ -1126,6 +1262,9 @@ gdm_session_worker_accredit_user (GdmSessionWorker  *worker,
                 goto out;
         }
 
+        gdm_session_auditor_report_user_accredited (worker->priv->auditor);
+
+        g_debug ("GdmSessionWorker: state ACCREDITED");
         worker->priv->state = GDM_SESSION_WORKER_STATE_ACCREDITED;
 
  out:
@@ -1274,8 +1413,7 @@ _open_session_log (const char *dir)
                         goto out;
                 }
 
-                g_warning (_("session log '%s' is not a normal file, "
-                             "logging session to '%s' instead.\n"), filename,
+                g_warning ("session log '%s' is not a normal file, logging session to '%s' instead.\n", filename,
                            temp_name);
                 g_free (filename);
                 filename = temp_name;
@@ -1298,13 +1436,32 @@ out:
         g_free (filename);
 
         if (fd < 0) {
-                g_warning (_("unable to log session"));
+                g_warning ("unable to log session");
                 fd = g_open ("/dev/null", O_RDWR);
         }
 
         return fd;
 }
 
+static void
+_save_user_settings (GdmSessionWorker *worker,
+                     const char       *home_dir)
+{
+        GError *error;
+
+        if (!gdm_session_settings_is_loaded (worker->priv->user_settings)) {
+                return;
+        }
+
+        error = NULL;
+        if (!gdm_session_settings_save (worker->priv->user_settings,
+                                        home_dir, &error)) {
+                g_warning ("could not save session and language settings: %s",
+                           error->message);
+                g_error_free (error);
+        }
+}
+
 static gboolean
 gdm_session_worker_start_user_session (GdmSessionWorker  *worker,
                                        GError           **error)
@@ -1367,6 +1524,8 @@ gdm_session_worker_start_user_session (GdmSessionWorker  *worker,
                 dup2 (fd, STDERR_FILENO);
                 close (fd);
 
+                _save_user_settings (worker, home_dir);
+
                 gdm_session_execute (worker->priv->arguments[0],
                                      worker->priv->arguments,
                                      environment,
@@ -1385,6 +1544,7 @@ gdm_session_worker_start_user_session (GdmSessionWorker  *worker,
         g_debug ("GdmSessionWorker: session opened creating reply...");
         g_assert (sizeof (GPid) <= sizeof (int));
 
+        g_debug ("GdmSessionWorker: state SESSION_STARTED");
         worker->priv->state = GDM_SESSION_WORKER_STATE_SESSION_STARTED;
 
         gdm_session_worker_watch_child (worker);
@@ -1417,6 +1577,7 @@ gdm_session_worker_open_user_session (GdmSessionWorker  *worker,
                 goto out;
         }
 
+        g_debug ("GdmSessionWorker: state SESSION_OPENED");
         worker->priv->state = GDM_SESSION_WORKER_STATE_SESSION_OPENED;
 
  out:
@@ -1425,6 +1586,8 @@ gdm_session_worker_open_user_session (GdmSessionWorker  *worker,
                 return FALSE;
         }
 
+        gdm_session_auditor_report_login (worker->priv->auditor);
+
         return TRUE;
 }
 
@@ -1500,18 +1663,130 @@ on_set_environment_variable (GdmSessionWorker *worker,
         }
 }
 
+static void
+gdm_session_worker_set_session_name (GdmSessionWorker *worker,
+                                     const char       *session_name)
+{
+        gdm_session_settings_set_session_name (worker->priv->user_settings,
+                                               session_name);
+}
+
+static void
+on_set_session_name (GdmSessionWorker *worker,
+                     DBusMessage      *message)
+{
+        DBusError   error;
+        const char *session_name;
+        dbus_bool_t res;
+
+        dbus_error_init (&error);
+        res = dbus_message_get_args (message,
+                                     &error,
+                                     DBUS_TYPE_STRING, &session_name,
+                                     DBUS_TYPE_INVALID);
+        if (res) {
+                g_debug ("GdmSessionWorker: session name set to %s", session_name);
+                gdm_session_worker_set_session_name (worker, session_name);
+        } else {
+                g_warning ("Unable to get arguments: %s", error.message);
+                dbus_error_free (&error);
+        }
+}
+
+static void
+gdm_session_worker_set_language_name (GdmSessionWorker *worker,
+                                      const char       *language_name)
+{
+        gdm_session_settings_set_language_name (worker->priv->user_settings,
+                                                language_name);
+}
+
+static void
+on_set_language_name (GdmSessionWorker *worker,
+                      DBusMessage      *message)
+{
+        DBusError   error;
+        const char *language_name;
+        dbus_bool_t res;
+
+        dbus_error_init (&error);
+        res = dbus_message_get_args (message,
+                                     &error,
+                                     DBUS_TYPE_STRING, &language_name,
+                                     DBUS_TYPE_INVALID);
+        if (res) {
+                g_debug ("GdmSessionWorker: language name set to %s", language_name);
+                gdm_session_worker_set_language_name (worker, language_name);
+        } else {
+                g_warning ("Unable to get arguments: %s", error.message);
+                dbus_error_free (&error);
+        }
+}
+
+static void
+on_saved_language_name_read (GdmSessionWorker *worker)
+{
+        char *language_name;
+
+        language_name = gdm_session_settings_get_language_name (worker->priv->user_settings);
+        send_dbus_string_method (worker->priv->connection,
+                                 "SavedLanguageNameRead",
+                                 language_name);
+        g_free (language_name);
+}
+
+static void
+on_saved_session_name_read (GdmSessionWorker *worker)
+{
+        char *session_name;
+
+        session_name = gdm_session_settings_get_session_name (worker->priv->user_settings);
+        send_dbus_string_method (worker->priv->connection,
+                                 "SavedSessionNameRead",
+                                 session_name);
+        g_free (session_name);
+}
+
 static void
 do_setup (GdmSessionWorker *worker)
 {
         GError  *error;
         gboolean res;
 
+        worker->priv->user_settings = gdm_session_settings_new ();
+
+        g_signal_connect_swapped (worker->priv->user_settings,
+                                  "notify::language-name",
+                                  G_CALLBACK (on_saved_language_name_read),
+                                  worker);
+
+        g_signal_connect_swapped (worker->priv->user_settings,
+                                  "notify::session-name",
+                                  G_CALLBACK (on_saved_session_name_read),
+                                  worker);
+
+        /* In some setups the user can read ~/.dmrc at this point.
+         * In some other setups the user can only read ~/.dmrc after completing
+         * the pam conversation.
+         *
+         * The user experience is better if we can read .dmrc now since we can
+         * prefill in the language and session combo boxes in the greeter with
+         * the right values.
+         *
+         * We'll try now, and if it doesn't work out, try later.
+         */
+        if (worker->priv->username != NULL) {
+                attempt_to_load_user_settings_as_root (worker,
+                                                       worker->priv->username);
+        }
+
         error = NULL;
         res = gdm_session_worker_initialize_pam (worker,
                                                  worker->priv->service,
                                                  worker->priv->username,
                                                  worker->priv->hostname,
                                                  worker->priv->x11_display_name,
+                                                 worker->priv->x11_authority_file,
                                                  worker->priv->display_device,
                                                  &error);
         if (! res) {
@@ -1640,13 +1915,51 @@ do_start_session (GdmSessionWorker *worker)
                               (int)worker->priv->child_pid);
 }
 
+static const char *
+get_state_name (int state)
+{
+        const char *name;
+
+        name = NULL;
+
+        switch (state) {
+        case GDM_SESSION_WORKER_STATE_NONE:
+                name = "NONE";
+                break;
+        case GDM_SESSION_WORKER_STATE_SETUP_COMPLETE:
+                name = "SETUP_COMPLETE";
+                break;
+        case GDM_SESSION_WORKER_STATE_AUTHENTICATED:
+                name = "AUTHENTICATED";
+                break;
+        case GDM_SESSION_WORKER_STATE_AUTHORIZED:
+                name = "AUTHORIZED";
+                break;
+        case GDM_SESSION_WORKER_STATE_ACCREDITED:
+                name = "ACCREDITED";
+                break;
+        case GDM_SESSION_WORKER_STATE_SESSION_OPENED:
+                name = "SESSION_OPENED";
+                break;
+        case GDM_SESSION_WORKER_STATE_SESSION_STARTED:
+                name = "SESSION_STARTED";
+                break;
+        default:
+                g_assert_not_reached ();
+                break;
+        }
+
+        return name;
+}
+
 static gboolean
 state_change_idle (GdmSessionWorker *worker)
 {
         int new_state;
 
         new_state = worker->priv->state + 1;
-        g_debug ("GdmSessionWorker: changing state to %d", new_state);
+        g_debug ("GdmSessionWorker: changing state to %s",
+                 get_state_name (new_state));
 
         worker->priv->state_change_idle_id = 0;
 
@@ -1731,6 +2044,7 @@ on_setup (GdmSessionWorker *worker,
         DBusError   error;
         const char *service;
         const char *x11_display_name;
+        const char *x11_authority_file;
         const char *console;
         const char *hostname;
         dbus_bool_t res;
@@ -1744,10 +2058,12 @@ on_setup (GdmSessionWorker *worker,
                                      DBUS_TYPE_STRING, &x11_display_name,
                                      DBUS_TYPE_STRING, &console,
                                      DBUS_TYPE_STRING, &hostname,
+                                     DBUS_TYPE_STRING, &x11_authority_file,
                                      DBUS_TYPE_INVALID);
         if (res) {
                 worker->priv->service = g_strdup (service);
                 worker->priv->x11_display_name = g_strdup (x11_display_name);
+                worker->priv->x11_authority_file = g_strdup (x11_authority_file);
                 worker->priv->display_device = g_strdup (console);
                 worker->priv->hostname = g_strdup (hostname);
                 worker->priv->username = NULL;
@@ -1767,6 +2083,7 @@ on_setup_for_user (GdmSessionWorker *worker,
         DBusError   error;
         const char *service;
         const char *x11_display_name;
+        const char *x11_authority_file;
         const char *console;
         const char *hostname;
         const char *username;
@@ -1781,11 +2098,13 @@ on_setup_for_user (GdmSessionWorker *worker,
                                      DBUS_TYPE_STRING, &x11_display_name,
                                      DBUS_TYPE_STRING, &console,
                                      DBUS_TYPE_STRING, &hostname,
+                                     DBUS_TYPE_STRING, &x11_authority_file,
                                      DBUS_TYPE_STRING, &username,
                                      DBUS_TYPE_INVALID);
         if (res) {
                 worker->priv->service = g_strdup (service);
                 worker->priv->x11_display_name = g_strdup (x11_display_name);
+                worker->priv->x11_authority_file = g_strdup (x11_authority_file);
                 worker->priv->display_device = g_strdup (console);
                 worker->priv->hostname = g_strdup (hostname);
                 worker->priv->username = g_strdup (username);
@@ -1871,6 +2190,10 @@ worker_dbus_handle_message (DBusConnection *connection,
                 on_start_program (worker, message);
         } else if (dbus_message_is_signal (message, GDM_SESSION_DBUS_INTERFACE, "SetEnvironmentVariable")) {
                 on_set_environment_variable (worker, message);
+        } else if (dbus_message_is_signal (message, GDM_SESSION_DBUS_INTERFACE, "SetLanguageName")) {
+                on_set_language_name (worker, message);
+        } else if (dbus_message_is_signal (message, GDM_SESSION_DBUS_INTERFACE, "SetSessionName")) {
+                on_set_session_name (worker, message);
         } else {
                 return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
         }
@@ -1918,11 +2241,8 @@ gdm_session_worker_constructor (GType                  type,
                                 GObjectConstructParam *construct_properties)
 {
         GdmSessionWorker      *worker;
-        GdmSessionWorkerClass *klass;
         DBusError              error;
 
-        klass = GDM_SESSION_WORKER_CLASS (g_type_class_peek (GDM_TYPE_SESSION_WORKER));
-
         worker = GDM_SESSION_WORKER (G_OBJECT_CLASS (gdm_session_worker_parent_class)->constructor (type,
                                                                                                     n_construct_properties,
                                                                                                     construct_properties));
