@@ -49,6 +49,8 @@ enum {
         SESSION_EXITED,
         SESSION_DIED,
         SELECTED_USER_CHANGED,
+        DEFAULT_LANGUAGE_NAME_CHANGED,
+        DEFAULT_SESSION_NAME_CHANGED,
         LAST_SIGNAL
 };
 
@@ -91,20 +93,22 @@ gdm_session_close (GdmSession *session)
 }
 
 void
-gdm_session_setup (GdmSession *session)
+gdm_session_setup (GdmSession *session,
+                   const char *service_name)
 {
         g_return_if_fail (GDM_IS_SESSION (session));
 
-        GDM_SESSION_GET_IFACE (session)->setup (session);
+        GDM_SESSION_GET_IFACE (session)->setup (session, service_name);
 }
 
 void
 gdm_session_setup_for_user (GdmSession *session,
+                            const char *service_name,
                             const char *username)
 {
         g_return_if_fail (GDM_IS_SESSION (session));
 
-        GDM_SESSION_GET_IFACE (session)->setup_for_user (session, username);
+        GDM_SESSION_GET_IFACE (session)->setup_for_user (session, service_name, username);
 }
 
 void
@@ -413,7 +417,28 @@ gdm_session_class_init (gpointer g_iface)
                               G_TYPE_NONE,
                               1,
                               G_TYPE_STRING);
-
+        signals [DEFAULT_LANGUAGE_NAME_CHANGED] =
+                g_signal_new ("default-language-name-changed",
+                              iface_type,
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GdmSessionIface, default_language_name_changed),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__STRING,
+                              G_TYPE_NONE,
+                              1,
+                              G_TYPE_STRING);
+        signals [DEFAULT_SESSION_NAME_CHANGED] =
+                g_signal_new ("default-session-name-changed",
+                              iface_type,
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GdmSessionIface, default_session_name_changed),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__STRING,
+                              G_TYPE_NONE,
+                              1,
+                              G_TYPE_STRING);
 }
 
 void
@@ -573,6 +598,24 @@ _gdm_session_closed (GdmSession   *session)
         g_signal_emit (session, signals [CLOSED], 0);
 }
 
+void
+_gdm_session_default_language_name_changed (GdmSession   *session,
+                                            const char   *language_name)
+{
+        g_return_if_fail (GDM_IS_SESSION (session));
+
+        g_signal_emit (session, signals [DEFAULT_LANGUAGE_NAME_CHANGED], 0, language_name);
+}
+
+void
+_gdm_session_default_session_name_changed (GdmSession   *session,
+                                           const char   *session_name)
+{
+        g_return_if_fail (GDM_IS_SESSION (session));
+
+        g_signal_emit (session, signals [DEFAULT_SESSION_NAME_CHANGED], 0, session_name);
+}
+
 void
 _gdm_session_selected_user_changed (GdmSession   *session,
                                     const char   *text)
