@@ -46,8 +46,10 @@ struct _GdmSessionIface
 
         /* Methods */
         void (* open)                        (GdmSession   *session);
-        void (* setup)                       (GdmSession   *session);
+        void (* setup)                       (GdmSession   *session,
+                                              const char   *service_name);
         void (* setup_for_user)              (GdmSession   *session,
+                                              const char   *service_name,
                                               const char   *username);
         void (* reset)                       (GdmSession   *session);
         void (* authenticate)                (GdmSession   *session);
@@ -103,13 +105,19 @@ struct _GdmSessionIface
         void (* selected_user_changed)       (GdmSession   *session,
                                               const char   *text);
 
+        void (* default_language_name_changed)    (GdmSession   *session,
+                                                   const char   *text);
+        void (* default_session_name_changed)     (GdmSession   *session,
+                                                   const char   *text);
 };
 
 GType    gdm_session_get_type                    (void) G_GNUC_CONST;
 
 void     gdm_session_open                        (GdmSession *session);
-void     gdm_session_setup                       (GdmSession *session);
+void     gdm_session_setup                       (GdmSession *session,
+                                                  const char *service_name);
 void     gdm_session_setup_for_user              (GdmSession *session,
+                                                  const char *service_name,
                                                   const char *username);
 void     gdm_session_reset                       (GdmSession *session);
 void     gdm_session_authenticate                (GdmSession *session);
