@@ -63,7 +63,6 @@
 
 struct GdmSimpleSlavePrivate
 {
-        char              *id;
         GPid               pid;
 
         guint              greeter_reset_id;
@@ -78,6 +77,9 @@ struct GdmSimpleSlavePrivate
         GdmGreeterServer  *greeter_server;
         GdmGreeterSession *greeter;
         GdmSessionDirect  *session;
+
+        guint              start_session_when_ready : 1;
+        guint              waiting_to_start_session : 1;
 };
 
 enum {
@@ -204,15 +206,23 @@ on_session_authentication_failed (GdmSession     *session,
         queue_greeter_reset (slave);
 }
 
+static void
+gdm_simple_slave_accredit_when_ready (GdmSimpleSlave *slave)
+{
+        if (slave->priv->start_session_when_ready) {
+                gdm_session_accredit (GDM_SESSION (slave->priv->session),
+                                      GDM_SESSION_CRED_ESTABLISH);
+        } else {
+                slave->priv->waiting_to_start_session = TRUE;
+        }
+}
+
 static void
 on_session_authorized (GdmSession     *session,
                        GdmSimpleSlave *slave)
 {
-        int flag;
-
-        flag = GDM_SESSION_CRED_ESTABLISH;
-
-        gdm_session_accredit (session, flag);
+        gdm_greeter_server_user_authorized (slave->priv->greeter_server);
+        gdm_simple_slave_accredit_when_ready (slave);
 }
 
 static void
@@ -353,17 +363,50 @@ on_session_secret_info_query (GdmSession     *session,
         gdm_greeter_server_secret_info_query (slave->priv->greeter_server, text);
 }
 
+static gboolean
+get_timed_login_details (GdmSimpleSlave  *slave,
+                         char           **username,
+                         int             *delay)
+{
+        gboolean enabled;
+        gboolean res;
+
+        enabled = FALSE;
+        res = gdm_settings_client_get_boolean (GDM_KEY_TIMED_LOGIN_ENABLE, &enabled);
+        if (! enabled) {
+                return FALSE;
+        }
+
+        *username = NULL;
+        res = gdm_settings_client_get_string (GDM_KEY_TIMED_LOGIN_USER, username);
+
+        if (username == NULL) {
+                return FALSE;
+        }
+
+        res = gdm_settings_client_get_int (GDM_KEY_TIMED_LOGIN_DELAY, delay);
+
+        return TRUE;
+}
+
 static void
 on_session_opened (GdmSession     *session,
                    GdmSimpleSlave *slave)
 {
         gboolean res;
+        char    *username;
+        int      delay;
 
         g_debug ("GdmSimpleSlave: session opened");
         res = gdm_greeter_server_ready (slave->priv->greeter_server);
         if (! res) {
                 g_warning ("Unable to send ready");
         }
+
+        if (get_timed_login_details (slave, &username, &delay)) {
+                gdm_greeter_server_request_timed_login (slave->priv->greeter_server, username, delay);
+                g_free (username);
+        }
 }
 
 static void
@@ -400,6 +443,7 @@ static void
 create_new_session (GdmSimpleSlave *slave)
 {
         gboolean       display_is_local;
+        char          *display_id;
         char          *display_name;
         char          *display_hostname;
         char          *display_device;
@@ -408,6 +452,7 @@ create_new_session (GdmSimpleSlave *slave)
         g_debug ("GdmSimpleSlave: Creating new session");
 
         g_object_get (slave,
+                      "display-id", &display_id,
                       "display-name", &display_name,
                       "display-hostname", &display_hostname,
                       "display-is-local", &display_is_local,
@@ -419,11 +464,13 @@ create_new_session (GdmSimpleSlave *slave)
                 display_device = gdm_server_get_display_device (slave->priv->server);
         }
 
-        slave->priv->session = gdm_session_direct_new (display_name,
+        slave->priv->session = gdm_session_direct_new (display_id,
+                                                       display_name,
                                                        display_hostname,
                                                        display_device,
                                                        display_x11_authority_file,
                                                        display_is_local);
+        g_free (display_id);
         g_free (display_name);
         g_free (display_device);
         g_free (display_hostname);
@@ -565,29 +612,11 @@ on_greeter_begin_verification (GdmGreeterServer *greeter_server,
 }
 
 static void
-on_greeter_begin_timed_login (GdmGreeterServer *greeter_server,
-                              GdmSimpleSlave   *slave)
+on_greeter_begin_auto_login (GdmGreeterServer *greeter_server,
+                             const char       *username,
+                             GdmSimpleSlave   *slave)
 {
-        char    *username;
-        gboolean enabled;
-        gboolean res;
-
-        g_debug ("GdmSimpleSlave: begin timed login");
-
-        enabled = FALSE;
-        res = gdm_settings_client_get_boolean (GDM_KEY_TIMED_LOGIN_ENABLE, &enabled);
-        if (! enabled) {
-                g_warning ("GdmSimpleSlave: timed login requested but disabled in configuration");
-                return;
-        }
-
-        username = NULL;
-        res = gdm_settings_client_get_string (GDM_KEY_TIMED_LOGIN_USER, &username);
-        if (username == NULL) {
-                g_warning ("GdmSimpleSlave: timed login requested but username not specified in configuration");
-                return;
-        }
-
+        g_debug ("GdmSimpleSlave: begin auto login for user '%s'", username);
         gdm_session_setup_for_user (GDM_SESSION (slave->priv->session),
                                     "gdm-autologin",
                                     username);
@@ -672,6 +701,28 @@ on_greeter_connected (GdmGreeterServer *greeter_server,
         }
 }
 
+static void
+on_start_session_when_ready (GdmGreeterServer *session,
+                             GdmSimpleSlave   *slave)
+{
+        g_debug ("GdmSimpleSlave: Will start session when ready");
+        slave->priv->start_session_when_ready = TRUE;
+
+        if (slave->priv->waiting_to_start_session) {
+                gdm_simple_slave_accredit_when_ready (slave);
+        }
+}
+
+static void
+on_start_session_later (GdmGreeterServer *session,
+                        GdmSimpleSlave   *slave)
+{
+        g_debug ("GdmSimpleSlave: Will start session when ready and told");
+        slave->priv->start_session_when_ready = FALSE;
+}
+
+
+
 static void
 setup_server (GdmSimpleSlave *slave)
 {
@@ -727,8 +778,8 @@ run_greeter (GdmSimpleSlave *slave)
 
         slave->priv->greeter_server = gdm_greeter_server_new (display_id);
         g_signal_connect (slave->priv->greeter_server,
-                          "begin-timed-login",
-                          G_CALLBACK (on_greeter_begin_timed_login),
+                          "begin-auto-login",
+                          G_CALLBACK (on_greeter_begin_auto_login),
                           slave);
         g_signal_connect (slave->priv->greeter_server,
                           "begin-verification",
@@ -762,6 +813,16 @@ run_greeter (GdmSimpleSlave *slave)
                           "cancelled",
                           G_CALLBACK (on_greeter_cancel),
                           slave);
+        g_signal_connect (slave->priv->greeter_server,
+                          "start-session-when-ready",
+                          G_CALLBACK (on_start_session_when_ready),
+                          slave);
+
+        g_signal_connect (slave->priv->greeter_server,
+                          "start-session-later",
+                          G_CALLBACK (on_start_session_later),
+                          slave);
+
         gdm_greeter_server_start (slave->priv->greeter_server);
 
         address = gdm_greeter_server_get_address (slave->priv->greeter_server);
