@@ -34,6 +34,7 @@
 #include <dbus/dbus-glib-lowlevel.h>
 
 #include "gdm-greeter-client.h"
+#include "gdm-marshal.h"
 
 #define GDM_GREETER_CLIENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_GREETER_CLIENT, GdmGreeterClientPrivate))
 
@@ -64,6 +65,10 @@ enum {
         READY,
         RESET,
         SELECTED_USER_CHANGED,
+        DEFAULT_LANGUAGE_NAME_CHANGED,
+        DEFAULT_SESSION_NAME_CHANGED,
+        TIMED_LOGIN_REQUESTED,
+        USER_AUTHORIZED,
         LAST_SIGNAL
 };
 
@@ -96,6 +101,36 @@ gdm_greeter_client_get_display_is_local (GdmGreeterClient *client)
         return client->priv->display_is_local;
 }
 
+static void
+emit_string_and_int_signal_for_message (GdmGreeterClient *client,
+                                        const char       *name,
+                                        DBusMessage      *message,
+                                        int               signal)
+{
+        DBusError   error;
+        const char *text;
+        int         number;
+        dbus_bool_t res;
+
+        dbus_error_init (&error);
+        res = dbus_message_get_args (message,
+                                     &error,
+                                     DBUS_TYPE_STRING, &text,
+                                     DBUS_TYPE_INT32, &number,
+                                     DBUS_TYPE_INVALID);
+        if (res) {
+
+                g_debug ("GdmGreeterClient: Received %s (%s %d)", name, text, number);
+
+                g_signal_emit (client,
+                               gdm_greeter_client_signals[signal],
+                               0, text, number);
+        } else {
+                g_warning ("Unable to get arguments: %s", error.message);
+                dbus_error_free (&error);
+        }
+}
+
 static void
 emit_string_signal_for_message (GdmGreeterClient *client,
                                 const char       *name,
@@ -113,7 +148,7 @@ emit_string_signal_for_message (GdmGreeterClient *client,
                                      DBUS_TYPE_INVALID);
         if (res) {
 
-                g_debug ("GdmGreeterClient: Recieved %s (%s)", name, text);
+                g_debug ("GdmGreeterClient: Received %s (%s)", name, text);
 
                 g_signal_emit (client,
                                gdm_greeter_client_signals[signal],
@@ -131,6 +166,36 @@ on_selected_user_changed (GdmGreeterClient *client,
         emit_string_signal_for_message (client, "SelectedUserChanged", message, SELECTED_USER_CHANGED);
 }
 
+static void
+on_default_language_name_changed (GdmGreeterClient *client,
+                                  DBusMessage      *message)
+{
+        emit_string_signal_for_message (client, "DefaultLanguageNameChanged", message, DEFAULT_LANGUAGE_NAME_CHANGED);
+}
+
+static void
+on_default_session_name_changed (GdmGreeterClient *client,
+                                 DBusMessage      *message)
+{
+        emit_string_signal_for_message (client, "DefaultSessionNameChanged", message, DEFAULT_SESSION_NAME_CHANGED);
+}
+
+static void
+on_timed_login_requested (GdmGreeterClient *client,
+                          DBusMessage      *message)
+{
+        emit_string_and_int_signal_for_message (client, "TimedLoginRequested", message, TIMED_LOGIN_REQUESTED);
+}
+
+static void
+on_user_authorized (GdmGreeterClient *client,
+                    DBusMessage      *message)
+{
+        g_signal_emit (client,
+                       gdm_greeter_client_signals[USER_AUTHORIZED],
+                       0);
+}
+
 static void
 on_info_query (GdmGreeterClient *client,
                DBusMessage      *message)
@@ -234,6 +299,51 @@ send_dbus_string_method (DBusConnection *connection,
         return TRUE;
 }
 
+static gboolean
+send_dbus_bool_method (DBusConnection *connection,
+                       const char     *method,
+                       gboolean        payload)
+{
+        DBusError       error;
+        DBusMessage    *message;
+        DBusMessage    *reply;
+        DBusMessageIter iter;
+
+        g_debug ("GdmGreeterClient: Calling %s", method);
+        message = dbus_message_new_method_call (NULL,
+                                                GREETER_SERVER_DBUS_PATH,
+                                                GREETER_SERVER_DBUS_INTERFACE,
+                                                method);
+        if (message == NULL) {
+                g_warning ("Couldn't allocate the D-Bus message");
+                return FALSE;
+        }
+
+        dbus_message_iter_init_append (message, &iter);
+        dbus_message_iter_append_basic (&iter,
+                                        DBUS_TYPE_BOOLEAN,
+                                        &payload);
+
+        dbus_error_init (&error);
+        reply = dbus_connection_send_with_reply_and_block (connection,
+                                                           message,
+                                                           -1,
+                                                           &error);
+
+        dbus_message_unref (message);
+
+        if (dbus_error_is_set (&error)) {
+                g_warning ("%s %s raised: %s\n",
+                           method,
+                           error.name,
+                           error.message);
+                return FALSE;
+        }
+        dbus_message_unref (reply);
+        dbus_connection_flush (connection);
+
+        return TRUE;
+}
 
 static gboolean
 send_dbus_void_method (DBusConnection *connection,
@@ -274,6 +384,14 @@ send_dbus_void_method (DBusConnection *connection,
         return TRUE;
 }
 
+void
+gdm_greeter_client_call_begin_auto_login (GdmGreeterClient *client,
+                                          const char       *username)
+{
+        send_dbus_string_method (client->priv->connection,
+                                 "BeginAutoLogin", username);
+}
+
 void
 gdm_greeter_client_call_begin_verification (GdmGreeterClient *client)
 {
@@ -299,6 +417,15 @@ gdm_greeter_client_call_answer_query (GdmGreeterClient *client,
                                  text);
 }
 
+void
+gdm_greeter_client_call_start_session_when_ready  (GdmGreeterClient *client,
+                                                   gboolean          should_start_session)
+{
+        send_dbus_bool_method (client->priv->connection,
+                               "StartSessionWhenReady",
+                               should_start_session);
+}
+
 void
 gdm_greeter_client_call_select_session (GdmGreeterClient *client,
                                         const char       *text)
@@ -497,6 +624,14 @@ client_dbus_handle_message (DBusConnection *connection,
                 on_reset (client, message);
         } else if (dbus_message_is_signal (message, GREETER_SERVER_DBUS_INTERFACE, "SelectedUserChanged")) {
                 on_selected_user_changed (client, message);
+        } else if (dbus_message_is_signal (message, GREETER_SERVER_DBUS_INTERFACE, "DefaultLanguageNameChanged")) {
+                on_default_language_name_changed (client, message);
+        } else if (dbus_message_is_signal (message, GREETER_SERVER_DBUS_INTERFACE, "DefaultSessionNameChanged")) {
+                on_default_session_name_changed (client, message);
+        } else if (dbus_message_is_signal (message, GREETER_SERVER_DBUS_INTERFACE, "TimedLoginRequested")) {
+                on_timed_login_requested (client, message);
+        } else if (dbus_message_is_signal (message, GREETER_SERVER_DBUS_INTERFACE, "UserAuthorized")) {
+                on_user_authorized (client, message);
         } else {
                 return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
         }
@@ -605,10 +740,6 @@ gdm_greeter_client_set_property (GObject        *object,
                                  const GValue   *value,
                                  GParamSpec     *pspec)
 {
-        GdmGreeterClient *self;
-
-        self = GDM_GREETER_CLIENT (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -622,10 +753,6 @@ gdm_greeter_client_get_property (GObject        *object,
                                  GValue         *value,
                                  GParamSpec     *pspec)
 {
-        GdmGreeterClient *self;
-
-        self = GDM_GREETER_CLIENT (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -639,9 +766,6 @@ gdm_greeter_client_constructor (GType                  type,
                                  GObjectConstructParam *construct_properties)
 {
         GdmGreeterClient      *greeter_client;
-        GdmGreeterClientClass *klass;
-
-        klass = GDM_GREETER_CLIENT_CLASS (g_type_class_peek (GDM_TYPE_GREETER_CLIENT));
 
         greeter_client = GDM_GREETER_CLIENT (G_OBJECT_CLASS (gdm_greeter_client_parent_class)->constructor (type,
                                                                                                             n_construct_properties,
@@ -653,10 +777,6 @@ gdm_greeter_client_constructor (GType                  type,
 static void
 gdm_greeter_client_dispose (GObject *object)
 {
-        GdmGreeterClient *greeter_client;
-
-        greeter_client = GDM_GREETER_CLIENT (object);
-
         g_debug ("GdmGreeterClient: Disposing greeter_client");
 
         G_OBJECT_CLASS (gdm_greeter_client_parent_class)->dispose (object);
@@ -754,6 +874,47 @@ gdm_greeter_client_class_init (GdmGreeterClientClass *klass)
                               g_cclosure_marshal_VOID__STRING,
                               G_TYPE_NONE,
                               1, G_TYPE_STRING);
+        gdm_greeter_client_signals[DEFAULT_LANGUAGE_NAME_CHANGED] =
+                g_signal_new ("default-language-name-changed",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GdmGreeterClientClass, default_language_name_changed),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__STRING,
+                              G_TYPE_NONE,
+                              1, G_TYPE_STRING);
+        gdm_greeter_client_signals[DEFAULT_SESSION_NAME_CHANGED] =
+                g_signal_new ("default-session-name-changed",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GdmGreeterClientClass, default_session_name_changed),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__STRING,
+                              G_TYPE_NONE,
+                              1, G_TYPE_STRING);
+
+        gdm_greeter_client_signals[TIMED_LOGIN_REQUESTED] =
+                g_signal_new ("timed-login-requested",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GdmGreeterClientClass, default_session_name_changed),
+                              NULL,
+                              NULL,
+                              gdm_marshal_VOID__STRING_INT,
+                              G_TYPE_NONE,
+                              2, G_TYPE_STRING, G_TYPE_INT);
+
+        gdm_greeter_client_signals[USER_AUTHORIZED] =
+                g_signal_new ("user-authorized",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GdmGreeterClientClass, user_authorized),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE, 0);
 }
 
 static void
