@@ -46,13 +46,13 @@
 
 #include <gdk/gdkkeysyms.h>
 #include <gdk/gdkx.h>
+#include <X11/XKBlib.h>
 
 #include <gtk/gtk.h>
 
 #include <glade/glade-xml.h>
 #include <gconf/gconf-client.h>
 
-#define DBUS_API_SUBJECT_TO_CHANGE
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
 
@@ -88,10 +88,11 @@
 
 #define GLADE_XML_FILE       "gdm-greeter-login-window.glade"
 
-#define KEY_LOGO                    "/apps/gdm/simple-greeter/logo-icon-name"
-#define KEY_DISABLE_RESTART_BUTTONS "/apps/gdm/simple-greeter/disable-restart-buttons"
-#define TIMED_LOGIN_TIMEOUT_SEC  60
-
+#define KEY_GREETER_DIR             "/apps/gdm/simple-greeter"
+#define KEY_BANNER_MESSAGE_ENABLED  KEY_GREETER_DIR "/banner_message_enable"
+#define KEY_BANNER_MESSAGE_TEXT     KEY_GREETER_DIR "/banner_message_text"
+#define KEY_LOGO                    KEY_GREETER_DIR "/logo_icon_name"
+#define KEY_DISABLE_RESTART_BUTTONS KEY_GREETER_DIR "/disable_restart_buttons"
 #define GDM_GREETER_LOGIN_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_GREETER_LOGIN_WINDOW, GdmGreeterLoginWindowPrivate))
 
 enum {
@@ -109,28 +110,40 @@ struct GdmGreeterLoginWindowPrivate
 {
         GladeXML        *xml;
         GtkWidget       *user_chooser;
-        gboolean         display_is_local;
-        char            *timeformat;
+        GtkWidget       *auth_capslock_label;
+        GtkWidget       *auth_banner_label;
+        guint            display_is_local : 1;
+        guint            is_interactive : 1;
+        GConfClient     *client;
+
+        gboolean         caps_lock_on;
+
+        gboolean         banner_message_enabled;
+        guint            gconf_cnxn;
 
         gboolean         timed_login_enabled;
         guint            timed_login_delay;
+        char            *timed_login_username;
         guint            timed_login_timeout_id;
 
         guint            animation_timeout_id;
 
         guint            login_button_handler_id;
+        guint            start_session_handler_id;
 };
 
 enum {
         PROP_0,
         PROP_DISPLAY_IS_LOCAL,
+        PROP_IS_INTERACTIVE,
 };
 
 enum {
-        BEGIN_TIMED_LOGIN,
+        BEGIN_AUTO_LOGIN,
         BEGIN_VERIFICATION,
         BEGIN_VERIFICATION_FOR_USER,
         QUERY_ANSWER,
+        START_SESSION,
         USER_SELECTED,
         DISCONNECTED,
         CANCELLED,
@@ -145,6 +158,41 @@ static void     gdm_greeter_login_window_finalize     (GObject
 
 G_DEFINE_TYPE (GdmGreeterLoginWindow, gdm_greeter_login_window, GTK_TYPE_WINDOW)
 
+static void
+capslock_update (GdmGreeterLoginWindow *login_window,
+                 gboolean               is_on)
+{
+
+        login_window->priv->caps_lock_on = is_on;
+
+        if (login_window->priv->auth_capslock_label == NULL) {
+                return;
+        }
+
+        if (is_on) {
+                gtk_label_set_text (GTK_LABEL (login_window->priv->auth_capslock_label),
+                                    _("You have the Caps Lock key on."));
+        } else {
+                gtk_label_set_text (GTK_LABEL (login_window->priv->auth_capslock_label),
+                                    "");
+        }
+}
+
+static gboolean
+is_capslock_on (void)
+{
+        XkbStateRec states;
+        Display     *dsp;
+
+        dsp = GDK_DISPLAY ();
+
+        if (XkbGetState (dsp, XkbUseCoreKbd, &states) != Success) {
+              return FALSE;
+        }
+
+        return (states.locked_mods & LockMask) != 0;
+}
+
 static void
 set_busy (GdmGreeterLoginWindow *login_window)
 {
@@ -212,14 +260,25 @@ remove_timed_login_timeout (GdmGreeterLoginWindow *login_window)
         }
 }
 
+static void
+_gdm_greeter_login_window_set_interactive (GdmGreeterLoginWindow *login_window,
+                                           gboolean               is_interactive)
+{
+
+        if (login_window->priv->is_interactive != is_interactive) {
+                login_window->priv->is_interactive = is_interactive;
+                g_object_notify (G_OBJECT (login_window), "is-interactive");
+        }
+}
+
 static gboolean
 timed_login_timer (GdmGreeterLoginWindow *login_window)
 {
         set_sensitive (login_window, FALSE);
         set_message (login_window, _("Automatically logging in..."));
 
-        g_debug ("GdmGreeterLoginWindow: emitting begin-timed_login");
-        g_signal_emit (login_window, signals[BEGIN_TIMED_LOGIN], 0);
+        g_debug ("GdmGreeterLoginWindow: timer expired");
+        _gdm_greeter_login_window_set_interactive (login_window, TRUE);
         login_window->priv->timed_login_timeout_id = 0;
 
         return FALSE;
@@ -236,6 +295,10 @@ restart_timed_login_timeout (GdmGreeterLoginWindow *login_window)
                 login_window->priv->timed_login_timeout_id = g_timeout_add_seconds (login_window->priv->timed_login_delay,
                                                                                     (GSourceFunc)timed_login_timer,
                                                                                     login_window);
+
+                gdm_chooser_widget_set_item_timer (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser),
+                                                   GDM_USER_CHOOSER_USER_AUTO,
+                                                   login_window->priv->timed_login_delay * 1000);
         }
 }
 
@@ -260,17 +323,14 @@ static gboolean
 get_show_restart_buttons (GdmGreeterLoginWindow *login_window)
 {
         gboolean     show;
-        GConfClient *client;
         GError      *error;
 
-        client = gconf_client_get_default ();
         error = NULL;
-        show = ! gconf_client_get_bool (client, KEY_DISABLE_RESTART_BUTTONS, &error);
+        show = ! gconf_client_get_bool (login_window->priv->client, KEY_DISABLE_RESTART_BUTTONS, &error);
         if (error != NULL) {
                 g_debug ("GdmGreeterLoginWindow: unable to get disable-restart-buttons configuration: %s", error->message);
                 g_error_free (error);
         }
-        g_object_unref (client);
 
 #ifdef ENABLE_RBAC_SHUTDOWN
         {
@@ -303,17 +363,18 @@ on_login_button_clicked_answer_query (GtkButton             *button,
         entry = glade_xml_get_widget (login_window->priv->xml, "auth-prompt-entry");
         text = gtk_entry_get_text (GTK_ENTRY (entry));
 
+        _gdm_greeter_login_window_set_interactive (login_window, TRUE);
         g_signal_emit (login_window, signals[QUERY_ANSWER], 0, text);
 }
 
 static void
 on_login_button_clicked_timed_login (GtkButton             *button,
-                                   GdmGreeterLoginWindow *login_window)
+                                     GdmGreeterLoginWindow *login_window)
 {
         set_busy (login_window);
         set_sensitive (login_window, FALSE);
 
-        g_signal_emit (login_window, signals[BEGIN_TIMED_LOGIN], 0);
+        _gdm_greeter_login_window_set_interactive (login_window, TRUE);
 }
 
 static void
@@ -365,8 +426,6 @@ switch_mode (GdmGreeterLoginWindow *login_window,
 
         switch (number) {
         case MODE_SELECTION:
-                restart_timed_login_timeout (login_window);
-
                 set_log_in_button_mode (login_window, LOGIN_BUTTON_HIDDEN);
 
                 show_widget (login_window, "cancel-button", FALSE);
@@ -469,19 +528,41 @@ do_suspend (GdmGreeterLoginWindow *login_window)
 static void
 do_cancel (GdmGreeterLoginWindow *login_window)
 {
-
+        gdm_chooser_widget_set_item_timer (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser),
+                                           GDM_USER_CHOOSER_USER_AUTO, 0);
+        login_window->priv->timed_login_enabled = FALSE;
         gdm_user_chooser_widget_set_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), NULL);
 
+        if (login_window->priv->start_session_handler_id > 0) {
+                g_signal_handler_disconnect (login_window, login_window->priv->start_session_handler_id);
+                login_window->priv->start_session_handler_id = 0;
+        }
+
         switch_mode (login_window, MODE_SELECTION);
         set_busy (login_window);
         set_sensitive (login_window, FALSE);
         set_message (login_window, "");
+        remove_timed_login_timeout (login_window);
+        _gdm_greeter_login_window_set_interactive (login_window, FALSE);
 
         g_signal_emit (login_window, signals[CANCELLED], 0);
 
         set_ready (login_window);
 }
 
+static void
+delete_entry_text (GtkWidget *entry)
+{
+        const char *typed_text;
+        char       *null_text;
+
+        /* try to scrub out any secret info */
+        typed_text = gtk_entry_get_text (GTK_ENTRY (entry));
+        null_text = g_strnfill (strlen (typed_text) + 1, '\b');
+        gtk_entry_set_text (GTK_ENTRY (entry), null_text);
+        gtk_entry_set_text (GTK_ENTRY (entry), "");
+}
+
 static void
 reset_dialog (GdmGreeterLoginWindow *login_window)
 {
@@ -491,7 +572,9 @@ reset_dialog (GdmGreeterLoginWindow *login_window)
         g_debug ("GdmGreeterLoginWindow: Resetting dialog");
 
         entry = glade_xml_get_widget (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->xml, "auth-prompt-entry");
-        gtk_entry_set_text (GTK_ENTRY (entry), "");
+
+        delete_entry_text (entry);
+
         gtk_entry_set_visibility (GTK_ENTRY (entry), TRUE);
         set_message (login_window, "");
 
@@ -507,8 +590,6 @@ reset_dialog (GdmGreeterLoginWindow *login_window)
         set_sensitive (login_window, TRUE);
         set_ready (login_window);
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
-
-        restart_timed_login_timeout (login_window);
 }
 
 gboolean
@@ -559,6 +640,60 @@ gdm_greeter_login_window_problem (GdmGreeterLoginWindow *login_window,
         return TRUE;
 }
 
+void
+gdm_greeter_login_window_request_timed_login (GdmGreeterLoginWindow *login_window,
+                                              const char            *username,
+                                              int                    delay)
+{
+        g_return_if_fail (GDM_IS_GREETER_LOGIN_WINDOW (login_window));
+
+        g_debug ("GdmGreeterLoginWindow: requested automatic login for user '%s' in %d seconds", username, delay);
+
+        login_window->priv->timed_login_username = g_strdup (username);
+        login_window->priv->timed_login_delay = delay;
+
+        reset_dialog (login_window);
+        gdm_user_chooser_widget_set_show_auto_user (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), TRUE);
+        gdm_user_chooser_widget_set_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), GDM_USER_CHOOSER_USER_AUTO);
+}
+
+static void
+gdm_greeter_login_window_start_session_when_ready (GdmGreeterLoginWindow *login_window)
+{
+        if (login_window->priv->is_interactive) {
+                g_debug ("GdmGreeterLoginWindow: starting session");
+                g_signal_emit (login_window, signals[START_SESSION], 0);
+        } else {
+                g_debug ("GdmGreeterLoginWindow: not starting session since "
+                         "user hasn't had an opportunity to pick language "
+                         "and session yet.");
+
+                /* Call back when we're ready to go
+                 */
+                login_window->priv->start_session_handler_id =
+                    g_signal_connect (login_window, "notify::is-interactive",
+                                      G_CALLBACK (gdm_greeter_login_window_start_session_when_ready),
+                                      NULL);
+
+                /* FIXME: If the user wasn't asked any questions by pam but
+                 * pam still authorized them (passwd -d, or the questions got
+                 * asked on an external device) then we need to let them log in.
+                 * Right now we just log them in right away, but we really should
+                 * set a timer up like timed login (but shorter, say ~5 seconds),
+                 * so they can pick language/session.  Will need to refactor things
+                 * a bit so we can share code with timed login.
+                 */
+                if (!login_window->priv->timed_login_enabled) {
+
+                        g_debug ("GdmGreeterLoginWindow: Okay, we'll start the session anyway,"
+                                 "because the user isn't ever going to get an opportunity to"
+                                 "interact with session");
+                        _gdm_greeter_login_window_set_interactive (login_window, TRUE);
+                }
+
+        }
+}
+
 gboolean
 gdm_greeter_login_window_info_query (GdmGreeterLoginWindow *login_window,
                                      const char            *text)
@@ -571,7 +706,7 @@ gdm_greeter_login_window_info_query (GdmGreeterLoginWindow *login_window,
         g_debug ("GdmGreeterLoginWindow: info query: %s", text);
 
         entry = glade_xml_get_widget (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->xml, "auth-prompt-entry");
-        gtk_entry_set_text (GTK_ENTRY (entry), "");
+        delete_entry_text (entry);
         gtk_entry_set_visibility (GTK_ENTRY (entry), TRUE);
         set_log_in_button_mode (login_window, LOGIN_BUTTON_ANSWER_QUERY);
 
@@ -596,7 +731,7 @@ gdm_greeter_login_window_secret_info_query (GdmGreeterLoginWindow *login_window,
         g_return_val_if_fail (GDM_IS_GREETER_LOGIN_WINDOW (login_window), FALSE);
 
         entry = glade_xml_get_widget (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->xml, "auth-prompt-entry");
-        gtk_entry_set_text (GTK_ENTRY (entry), "");
+        delete_entry_text (entry);
         gtk_entry_set_visibility (GTK_ENTRY (entry), FALSE);
         set_log_in_button_mode (login_window, LOGIN_BUTTON_ANSWER_QUERY);
 
@@ -611,6 +746,16 @@ gdm_greeter_login_window_secret_info_query (GdmGreeterLoginWindow *login_window,
         return TRUE;
 }
 
+void
+gdm_greeter_login_window_user_authorized (GdmGreeterLoginWindow *login_window)
+{
+        g_return_if_fail (GDM_IS_GREETER_LOGIN_WINDOW (login_window));
+
+        g_debug ("GdmGreeterLoginWindow: user now authorized");
+
+        gdm_greeter_login_window_start_session_when_ready (login_window);
+}
+
 static void
 _gdm_greeter_login_window_set_display_is_local (GdmGreeterLoginWindow *login_window,
                                                 gboolean               is)
@@ -632,6 +777,9 @@ gdm_greeter_login_window_set_property (GObject      *object,
         case PROP_DISPLAY_IS_LOCAL:
                 _gdm_greeter_login_window_set_display_is_local (self, g_value_get_boolean (value));
                 break;
+        case PROP_IS_INTERACTIVE:
+                _gdm_greeter_login_window_set_interactive (self, g_value_get_boolean (value));
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -652,6 +800,10 @@ gdm_greeter_login_window_get_property (GObject    *object,
         case PROP_DISPLAY_IS_LOCAL:
                 g_value_set_boolean (value, self->priv->display_is_local);
                 break;
+        case PROP_IS_INTERACTIVE:
+                g_value_set_boolean (value, self->priv->is_interactive);
+                break;
+
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -954,20 +1106,20 @@ on_user_chosen (GdmUserChooserWidget  *user_chooser,
                        0, user_name);
 
         if (strcmp (user_name, GDM_USER_CHOOSER_USER_OTHER) == 0) {
-                /* stop the timed login until the verification times
-                   out or is cancelled */
-                remove_timed_login_timeout (login_window);
                 g_signal_emit (login_window, signals[BEGIN_VERIFICATION], 0);
         } else if (strcmp (user_name, GDM_USER_CHOOSER_USER_GUEST) == 0) {
                 /* FIXME: handle guest account stuff */
         } else if (strcmp (user_name, GDM_USER_CHOOSER_USER_AUTO) == 0) {
+                g_signal_emit (login_window, signals[BEGIN_AUTO_LOGIN], 0,
+                               login_window->priv->timed_login_username);
+
+                login_window->priv->timed_login_enabled = TRUE;
+                restart_timed_login_timeout (login_window);
+
                 /* just wait for the user to select language and stuff */
                 set_log_in_button_mode (login_window, LOGIN_BUTTON_TIMED_LOGIN);
                 set_message (login_window, _("Select language and click Log In"));
         } else {
-                /* stop the timed login until the verification times
-                   out or is cancelled */
-                remove_timed_login_timeout (login_window);
                 g_signal_emit (login_window, signals[BEGIN_VERIFICATION_FOR_USER], 0, user_name);
         }
 
@@ -1151,19 +1303,18 @@ load_theme (GdmGreeterLoginWindow *login_window)
 
         image = glade_xml_get_widget (login_window->priv->xml, "logo-image");
         if (image != NULL) {
-                GConfClient *client;
                 char        *icon_name;
                 GError      *error;
 
-                client = gconf_client_get_default ();
                 error = NULL;
-                icon_name = gconf_client_get_string (client, KEY_LOGO, &error);
+                icon_name = gconf_client_get_string (login_window->priv->client, KEY_LOGO, &error);
                 if (error != NULL) {
                         g_debug ("GdmGreeterLoginWindow: unable to get logo icon name: %s", error->message);
                         g_error_free (error);
                 }
-                g_object_unref (client);
-                g_debug ("GdmGreeterLoginWindow: Got greeter logo '%s'", icon_name);
+
+                g_debug ("GdmGreeterLoginWindow: Got greeter logo '%s'",
+                          icon_name ? icon_name : "(null)");
                 if (icon_name != NULL) {
                         gtk_image_set_from_icon_name (GTK_IMAGE (image),
                                                       icon_name,
@@ -1184,10 +1335,6 @@ load_theme (GdmGreeterLoginWindow *login_window)
 
         gdm_user_chooser_widget_set_show_only_chosen (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), TRUE);
 
-        if (login_window->priv->timed_login_enabled) {
-                gdm_user_chooser_widget_set_show_auto_user (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), TRUE);
-        }
-
         g_signal_connect (login_window->priv->user_chooser,
                           "activated",
                           G_CALLBACK (on_user_chosen),
@@ -1199,6 +1346,11 @@ load_theme (GdmGreeterLoginWindow *login_window)
 
         gtk_widget_show (login_window->priv->user_chooser);
 
+        login_window->priv->auth_banner_label = glade_xml_get_widget (login_window->priv->xml, "auth-banner-label");
+        /*make_label_small_italic (login_window->priv->auth_banner_label);*/
+
+        login_window->priv->auth_capslock_label = glade_xml_get_widget (login_window->priv->xml, "auth-capslock-label");
+
         button = glade_xml_get_widget (login_window->priv->xml, "suspend-button");
         g_signal_connect (button, "clicked", G_CALLBACK (suspend_button_clicked), login_window);
 
@@ -1280,10 +1432,21 @@ static gboolean
 gdm_greeter_login_window_key_press_event (GtkWidget   *widget,
                                           GdkEventKey *event)
 {
+        GdmGreeterLoginWindow *login_window;
+        gboolean               capslock_on;
+
+        login_window = GDM_GREETER_LOGIN_WINDOW (widget);
+
         if (event->keyval == GDK_Escape) {
                 reset_dialog (GDM_GREETER_LOGIN_WINDOW (widget));
         }
 
+        capslock_on = is_capslock_on ();
+
+        if (capslock_on != login_window->priv->caps_lock_on) {
+                capslock_update (login_window, capslock_on);
+        }
+
         return GTK_WIDGET_CLASS (gdm_greeter_login_window_parent_class)->key_press_event (widget, event);
 }
 
@@ -1338,46 +1501,44 @@ gdm_greeter_login_window_size_allocate (GtkWidget      *widget,
 }
 
 static void
-read_configuration (GdmGreeterLoginWindow *login_window)
+update_banner_message (GdmGreeterLoginWindow *login_window)
 {
-        gboolean res;
-        int      delay;
-        char    *username;
+        GError      *error;
+        gboolean     enabled;
 
-        g_debug ("GdmGreeterLoginWindow: reading system configuration");
-
-        res = gdm_settings_client_get_boolean (GDM_KEY_TIMED_LOGIN_ENABLE,
-                                               &login_window->priv->timed_login_enabled);
-        if (! res) {
-                g_warning ("Unable to read configuration for %s", GDM_KEY_TIMED_LOGIN_ENABLE);
+        if (login_window->priv->auth_banner_label == NULL) {
+                /* if the theme doesn't have a banner message */
+                g_debug ("GdmGreeterLoginWindow: theme doesn't support a banner message");
+                return;
         }
-        g_debug ("GdmGreeterLoginWindow: TimedLoginEnable=%s",
-                 login_window->priv->timed_login_enabled ? "true" : "false");
 
-        /* treat failures here as disabling */
-        username = NULL;
-        res = gdm_settings_client_get_string (GDM_KEY_TIMED_LOGIN_USER, &username);
-        if (! res) {
-                g_warning ("Unable to read configuration for %s", GDM_KEY_TIMED_LOGIN_USER);
-                login_window->priv->timed_login_enabled = FALSE;
+        error = NULL;
+        enabled = gconf_client_get_bool (login_window->priv->client, KEY_BANNER_MESSAGE_ENABLED, &error);
+        if (error != NULL) {
+                g_debug ("GdmGreeterLoginWindow: unable to get configuration: %s", error->message);
+                g_error_free (error);
+        }
+
+        login_window->priv->banner_message_enabled = enabled;
+
+        if (! enabled) {
+                g_debug ("GdmGreeterLoginWindow: banner message disabled");
+                gtk_widget_hide (login_window->priv->auth_banner_label);
         } else {
-                if (username == NULL) {
-                        login_window->priv->timed_login_enabled = FALSE;
+                char *message;
+                error = NULL;
+                message = gconf_client_get_string (login_window->priv->client, KEY_BANNER_MESSAGE_TEXT, &error);
+                if (message != NULL) {
+                        char *markup;
+                        markup = g_markup_printf_escaped ("<small><i>%s</i></small>", message);
+                        gtk_label_set_markup (GTK_LABEL (login_window->priv->auth_banner_label),
+                                              markup);
+                        g_free (markup);
                 }
-        }
-        g_debug ("GdmGreeterLoginWindow: TimedLogin=%s", username);
-        g_free (username);
+                g_debug ("GdmGreeterLoginWindow: banner message: %s", message);
 
-        delay = -1;
-        res = gdm_settings_client_get_int (GDM_KEY_TIMED_LOGIN_DELAY, &delay);
-        if (! res) {
-                g_warning ("Unable to read configuration for %s", GDM_KEY_TIMED_LOGIN_DELAY);
-        } else {
-                if (delay >= 0) {
-                        login_window->priv->timed_login_delay = delay;
-                }
+                gtk_widget_show (login_window->priv->auth_banner_label);
         }
-        g_debug ("GdmGreeterLoginWindow: TimedLoginDelay=%d", delay);
 }
 
 static GObject *
@@ -1392,8 +1553,8 @@ gdm_greeter_login_window_constructor (GType                  type,
                                                                                                                       construct_properties));
 
 
-        read_configuration (login_window);
         load_theme (login_window);
+        update_banner_message (login_window);
 
         return G_OBJECT (login_window);
 }
@@ -1413,16 +1574,15 @@ gdm_greeter_login_window_class_init (GdmGreeterLoginWindowClass *klass)
         widget_class->size_request = gdm_greeter_login_window_size_request;
         widget_class->size_allocate = gdm_greeter_login_window_size_allocate;
 
-        signals [BEGIN_TIMED_LOGIN] =
-                g_signal_new ("begin-timed_login",
+        signals [BEGIN_AUTO_LOGIN] =
+                g_signal_new ("begin-auto-login",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GdmGreeterLoginWindowClass, begin_timed_login),
+                              G_STRUCT_OFFSET (GdmGreeterLoginWindowClass, begin_auto_login),
                               NULL,
                               NULL,
-                              g_cclosure_marshal_VOID__VOID,
-                              G_TYPE_NONE,
-                              0);
+                              g_cclosure_marshal_VOID__STRING,
+                              G_TYPE_NONE, 1, G_TYPE_STRING);
         signals [BEGIN_VERIFICATION] =
                 g_signal_new ("begin-verification",
                               G_TYPE_FROM_CLASS (object_class),
@@ -1483,6 +1643,17 @@ gdm_greeter_login_window_class_init (GdmGreeterLoginWindowClass *klass)
                               g_cclosure_marshal_VOID__VOID,
                               G_TYPE_NONE,
                               0);
+        signals [START_SESSION] =
+                g_signal_new ("start-session",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GdmGreeterLoginWindowClass, start_session),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0);
+
         g_object_class_install_property (object_class,
                                          PROP_DISPLAY_IS_LOCAL,
                                          g_param_spec_boolean ("display-is-local",
@@ -1490,17 +1661,56 @@ gdm_greeter_login_window_class_init (GdmGreeterLoginWindowClass *klass)
                                                                "display is local",
                                                                FALSE,
                                                                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+        g_object_class_install_property (object_class,
+                                         PROP_IS_INTERACTIVE,
+                                         g_param_spec_boolean ("is-interactive",
+                                                               "Is Interactive",
+                                                               "Use has had an oppurtunity to interact with window",
+                                                               FALSE,
+                                                               G_PARAM_READABLE));
 
         g_type_class_add_private (klass, sizeof (GdmGreeterLoginWindowPrivate));
 }
 
+static void
+on_gconf_key_changed (GConfClient           *client,
+                      guint                  cnxn_id,
+                      GConfEntry            *entry,
+                      GdmGreeterLoginWindow *login_window)
+{
+        const char *key;
+        GConfValue *value;
+
+        key = gconf_entry_get_key (entry);
+        value = gconf_entry_get_value (entry);
+
+        if (strcmp (key, KEY_BANNER_MESSAGE_ENABLED) == 0) {
+                if (value->type == GCONF_VALUE_BOOL) {
+                        gboolean enabled;
+
+                        enabled = gconf_value_get_bool (value);
+                        g_debug ("setting key %s = %d", key, enabled);
+                        login_window->priv->banner_message_enabled = enabled;
+                        update_banner_message (login_window);
+                } else {
+                        g_warning ("Error retrieving configuration key '%s': Invalid type",
+                                   key);
+                }
+        } else if (strcmp (key, KEY_BANNER_MESSAGE_TEXT) == 0) {
+                if (login_window->priv->banner_message_enabled) {
+                        update_banner_message (login_window);
+                }
+        } else {
+                g_debug ("Config key not handled: %s", key);
+        }
+}
+
 static void
 gdm_greeter_login_window_init (GdmGreeterLoginWindow *login_window)
 {
         login_window->priv = GDM_GREETER_LOGIN_WINDOW_GET_PRIVATE (login_window);
 
         login_window->priv->timed_login_enabled = FALSE;
-        login_window->priv->timed_login_delay = TIMED_LOGIN_TIMEOUT_SEC;
 
         gtk_window_set_title (GTK_WINDOW (login_window), _("Login Window"));
         gtk_window_set_opacity (GTK_WINDOW (login_window), 0.85);
@@ -1511,6 +1721,18 @@ gdm_greeter_login_window_init (GdmGreeterLoginWindow *login_window)
         gtk_window_set_skip_pager_hint (GTK_WINDOW (login_window), TRUE);
         gtk_window_stick (GTK_WINDOW (login_window));
         gtk_container_set_border_width (GTK_CONTAINER (login_window), 25);
+
+        login_window->priv->client = gconf_client_get_default ();
+        gconf_client_add_dir (login_window->priv->client,
+                              KEY_GREETER_DIR,
+                              GCONF_CLIENT_PRELOAD_ONELEVEL,
+                              NULL);
+        login_window->priv->gconf_cnxn = gconf_client_notify_add (login_window->priv->client,
+                                                                  KEY_GREETER_DIR,
+                                                                  (GConfClientNotifyFunc)on_gconf_key_changed,
+                                                                  login_window,
+                                                                  NULL,
+                                                                  NULL);
 }
 
 static void
@@ -1525,6 +1747,10 @@ gdm_greeter_login_window_finalize (GObject *object)
 
         g_return_if_fail (login_window->priv != NULL);
 
+        if (login_window->priv->client != NULL) {
+                g_object_unref (login_window->priv->client);
+        }
+
         G_OBJECT_CLASS (gdm_greeter_login_window_parent_class)->finalize (object);
 }
 
